Requirements

- Python 3.11
- Recommended: Create a virtual environment
  (e.g. venv: https://docs.python.org/3/library/venv.html,
  conda: https://conda.io/projects/conda/en/latest/user-guide/tasks/manage-environments.html)
- Install the package: https://pypi.org/project/besser-agentic-framework/

    pip install besser-agentic-framework[all]

To start the agent, run:
    python {{agent.name}}.py

In case your agent uses an LLM, you will need to configure the LLM credentials in the config.ini file.

Check out the official BAF repository and documentation for detailed information: 
https://github.com/BESSER-PEARL/BESSER-Agentic-Framework